**********  SAS Input Program for SAS Version for the PC  **********;
*NOTE: This SAS program converts raw-data files into SAS version 6.12
       or 8.x system files

* I M P O R T A N T :
       PLEASE NOTE THAT YOU MUST SPECIFY THE "<PATH>" FOR BOTH THE
       INPUT AND OUTPUT FILES

OPTION NOSOURCE REPLACE NOCENTER ;

%let cty = ARM AUS BFL COT CQU CYP ENG HKG HUN IRN
           ITA JPN LTU LVA MAR MDA NLD NOR NZL PHL
           RUS SCO SGP SVN TUN TWN USA UIN YEM ;

*NOTE: LIBNAME and FILENAME statements need to be modified ;

* library where the SAS data files are written to;
***YOU NEED TO ADAPT THE FOLLOWING LINE;
LIBNAME libdat "<PATH>" ;

%macro doit;
   %let i = 1;
      %do %while(%length(%scan(&cty,&i)));
	     %let ccode = %lowcase(%scan(&cty,&i));

%let fname= AST&ccode.m3;

*NOTE: file-specification of the input raw-data file ;
***YOU NEED TO ADAPT THE FOLLOWING LINE;
FILENAME rawinp "<PATH>\&fname..RAW" ;

%PUT INFO: data set definition ;
%PUT INFO: data set definitions have to be modified ;

*Output data set ;
data libdat.&fname;

*NOTE: LRECL may need to be modified ;
INFILE rawinp LRECL=1250 END=eof LENGTH=lnline LINE=ln MISSOVER ;

%PUT INFO: shorten variable definitions to save secondary storage area ;
%PUT INFO: refer to the SAS manual for an appropriate length for variables ;

*NOTE: The LENGTH statement in a data step specifies the number of bytes, the
       SAS - sytem uses for storing values of variables in each data set being created. ;
LENGTH DEFAULT=5 ;
LENGTH      IDCNTRY    5 ;
LENGTH      IDPOP      3 ;
LENGTH      IDSCHOOL   6 ;
LENGTH      IDCLASS    8 ;
LENGTH      IDSTUD     8 ;
LENGTH      IDTEACH    8 ;
LENGTH      IDLINK     3 ;
LENGTH      IDSTRATE   3 ;
LENGTH      IDSTRATI   3 ;
LENGTH      IDGRADE    3 ;
LENGTH      IDGRADER   3 ;
LENGTH      ITCOURSE   3 ;
LENGTH      MATSUBJ    3 ;
LENGTH      SCISUBJ    3 ;
LENGTH      NMTEACH    3 ;
LENGTH      NSTEACH    3 ;
LENGTH      NTEACH     3 ;
LENGTH      INTMS03    3 ;
LENGTH      INTMS99    3 ;
LENGTH      INTMS95    3 ;
LENGTH      MATWGT     8 ;
LENGTH      SCIWGT     8 ;
LENGTH      TCHWGT     8 ;
LENGTH      TOTWGT     8 ;
LENGTH      JKREP      3 ;
LENGTH      JKZONE     3 ;
LENGTH      ASMNRSC    8 ;
LENGTH      ASSNRSC    8 ;
LENGTH      ASMMAT01   8 ;
LENGTH      ASMMAT02   8 ;
LENGTH      ASMMAT03   8 ;
LENGTH      ASMMAT04   8 ;
LENGTH      ASMMAT05   8 ;
LENGTH      ASSSCI01   8 ;
LENGTH      ASSSCI02   8 ;
LENGTH      ASSSCI03   8 ;
LENGTH      ASSSCI04   8 ;
LENGTH      ASSSCI05   8 ;
LENGTH      DPCDATE    6 ;


%PUT INFO: Missing category A refers to "not administered" values ;
%PUT INFO: Missing category I refers to "invalid" values ;
%PUT INFO: Missing category B refers to "logically not applicable" values ;
%PUT INFO: Missing category R refers to "not reached" items ;
%PUT INFO: Missing category . refers to missing values ;
MISSING A I B R ;

*NOTE: The INPUT statement reads raw data and assigns variable names.
       The positions specified below correspond to the positions of the variables
       in the raw data file. ;

INPUT
  IDCNTRY       1-   5     
  IDPOP         6-   6     
  IDSCHOOL      7-  12     
  IDCLASS      13-  20     
  IDSTUD       21-  30     
  IDTEACH      31-  38     
  IDLINK       39-  40     
  IDSTRATE     41-  43     
  IDSTRATI     44-  46     
  IDGRADE      47-  48     
  IDGRADER     49-  49     
  ITCOURSE     50-  51     
  MATSUBJ      52-  52     
  SCISUBJ      53-  53     
  NMTEACH      54-  54     
  NSTEACH      55-  55     
  NTEACH       56-  56     
  INTMS03      57-  57     
  INTMS99      58-  58     
  INTMS95      59-  59     
  MATWGT       60-  72 .6
  SCIWGT       73-  85 .6
  TCHWGT       86-  98 .6
  TOTWGT       99- 111 .6
  JKREP       112- 112     
  JKZONE      113- 114     
  ASMNRSC     115- 122 .3
  ASSNRSC     123- 130 .3
  ASMMAT01    131- 136 .2
  ASMMAT02    137- 142 .2
  ASMMAT03    143- 148 .2
  ASMMAT04    149- 154 .2
  ASMMAT05    155- 160 .2
  ASSSCI01    161- 166 .2
  ASSSCI02    167- 172 .2
  ASSSCI03    173- 178 .2
  ASSSCI04    179- 184 .2
  ASSSCI05    185- 190 .2
  DPCDATE     191- 196     
;




%PUT INFO: assign labels to variables ;
LABEL
  IDCNTRY  = "*COUNTRY ID*"
  IDPOP    = "*POPULATION ID*"
  IDSCHOOL = "*SCHOOL ID*"
  IDCLASS  = "*CLASS ID*"
  IDSTUD   = "*STUDENT ID*"
  IDTEACH  = "*TEACHER ID*"
  IDLINK   = "*TEACHER LINK NUMBER*"
  IDSTRATE = "*EXPLICIT STRATUM CODE*"
  IDSTRATI = "*IMPLICIT STRATUM CODE*"
  IDGRADE  = "*GRADE*"
  IDGRADER = "*STANDARDIZED GRADE ID*"
  ITCOURSE = "*NAME OF COURSE*"
  MATSUBJ  = "*MATHEMATICS TEACHER LINK*"
  SCISUBJ  = "*SCIENCE TEACHER LINK*"
  NMTEACH  = "*NUMBER OF MATH TEACHERS FOR STUDENT*"
  NSTEACH  = "*NUMBER OF SCIENCE TEACHERS FOR STUDENT*"
  NTEACH   = "*NO. OF MATH+SCIENCE TEACHERS COMBINED*"
  INTMS03  = "*IN 2003 ASSESSMENT POPULATION*"
  INTMS99  = "*IN 2003 AND 1999 ASSESSMENT POPULATION*"
  INTMS95  = "*IN 2003 AND 1999 AND 1995 ASS. POP*"
  MATWGT   = "*WEIGHT FOR MATHEMATICS TEACHER DATA*"
  SCIWGT   = "*WEIGHT FOR SCIENCE TEACHER DATA*"
  TCHWGT   = "*WEIGHT FOR MAT+SCI TEACHER DATA COMBIN*"
  TOTWGT   = "*STUDENT WEIGHT*"
  JKREP    = "*REPLICATE CODE*"
  JKZONE   = "*ZONE CODE*"
  ASMNRSC  = "*NATIONAL MATH RASCH SCORE (150,10)*"
  ASSNRSC  = "*NATIONAL SCIENCE RASCH SCORE (150,10)*"
  ASMMAT01 = "*1ST PLAUSIBLE VALUE MATHEMATICS*"
  ASMMAT02 = "*2ND PLAUSIBLE VALUE MATHEMATICS*"
  ASMMAT03 = "*3RD PLAUSIBLE VALUE MATHEMATICS*"
  ASMMAT04 = "*4TH PLAUSIBLE VALUE MATHEMATICS*"
  ASMMAT05 = "*5TH PLAUSIBLE VALUE MATHEMATICS*"
  ASSSCI01 = "*1ST PLAUSIBLE VALUE SCIENCE*"
  ASSSCI02 = "*2ND PLAUSIBLE VALUE SCIENCE*"
  ASSSCI03 = "*3RD PLAUSIBLE VALUE SCIENCE*"
  ASSSCI04 = "*4TH PLAUSIBLE VALUE SCIENCE*"
  ASSSCI05 = "*5TH PLAUSIBLE VALUE SCIENCE*"
  DPCDATE  = "*FILE CREATION DATE AT DPC HAMBURG*"
;

%PUT INFO: recode "not administered" data to missing code ".A" ;
*NOTE: If a CHECK variable is coded correctly with a BLANK, it will be recoded
       to "1", otherwise it will be recoded to "0" ;

IF IDCNTRY  = 99998            THEN IDCNTRY  = .A ;
IF IDPOP    = 8                THEN IDPOP    = .A ;
IF IDSCHOOL = 999998           THEN IDSCHOOL = .A ;
IF IDCLASS  = 99999998         THEN IDCLASS  = .A ;
IF IDSTUD   = 9999999998       THEN IDSTUD   = .A ;
IF IDTEACH  = 99999998         THEN IDTEACH  = .A ;
IF IDLINK   = 98               THEN IDLINK   = .A ;
IF IDSTRATE = 998              THEN IDSTRATE = .A ;
IF IDSTRATI = 998              THEN IDSTRATI = .A ;
IF IDGRADER = 8                THEN IDGRADER = .A ;
IF ITCOURSE = 98               THEN ITCOURSE = .A ;
IF INTMS03  = 8                THEN INTMS03  = .A ;
IF INTMS99  = 8                THEN INTMS99  = .A ;
IF INTMS95  = 8                THEN INTMS95  = .A ;
IF MATWGT   = 999998           THEN MATWGT   = .A ;
IF SCIWGT   = 999998           THEN SCIWGT   = .A ;
IF TCHWGT   = 999998           THEN TCHWGT   = .A ;
IF TOTWGT   = 999998           THEN TOTWGT   = .A ;
IF JKREP    = 8                THEN JKREP    = .A ;
IF JKZONE   = 98               THEN JKZONE   = .A ;
IF ASMNRSC  = 9998             THEN ASMNRSC  = .A ;
IF ASSNRSC  = 9998             THEN ASSNRSC  = .A ;
IF ASMMAT01 = 998              THEN ASMMAT01 = .A ;
IF ASMMAT02 = 998              THEN ASMMAT02 = .A ;
IF ASMMAT03 = 998              THEN ASMMAT03 = .A ;
IF ASMMAT04 = 998              THEN ASMMAT04 = .A ;
IF ASMMAT05 = 998              THEN ASMMAT05 = .A ;
IF ASSSCI01 = 998              THEN ASSSCI01 = .A ;
IF ASSSCI02 = 998              THEN ASSSCI02 = .A ;
IF ASSSCI03 = 998              THEN ASSSCI03 = .A ;
IF ASSSCI04 = 998              THEN ASSSCI04 = .A ;
IF ASSSCI05 = 998              THEN ASSSCI05 = .A ;

%PUT INFO: recode special missing codes to SAS missing codes ;
*NOTE: Special missing codes are recoded to SAS
       character - missing codes;


%PUT INFO: recode omitted data to missing code "." ;
*NOTE: BLANKS are recoded to SYSMIS by SAS ;

IF IDCNTRY  = 99999            THEN IDCNTRY  = . ;
IF IDPOP    = 9                THEN IDPOP    = . ;
IF IDSCHOOL = 999999           THEN IDSCHOOL = . ;
IF IDCLASS  = 99999999         THEN IDCLASS  = . ;
IF IDSTUD   = 9999999999       THEN IDSTUD   = . ;
IF IDTEACH  = 99999999         THEN IDTEACH  = . ;
IF IDLINK   = 99               THEN IDLINK   = . ;
IF IDSTRATE = 999              THEN IDSTRATE = . ;
IF IDSTRATI = 999              THEN IDSTRATI = . ;
IF IDGRADER = 9                THEN IDGRADER = . ;
IF ITCOURSE = 99               THEN ITCOURSE = . ;
IF INTMS03  = 9                THEN INTMS03  = . ;
IF INTMS99  = 9                THEN INTMS99  = . ;
IF INTMS95  = 9                THEN INTMS95  = . ;
IF MATWGT   = 999999           THEN MATWGT   = . ;
IF SCIWGT   = 999999           THEN SCIWGT   = . ;
IF TCHWGT   = 999999           THEN TCHWGT   = . ;
IF TOTWGT   = 999999           THEN TOTWGT   = . ;
IF JKREP    = 9                THEN JKREP    = . ;
IF JKZONE   = 99               THEN JKZONE   = . ;
IF ASMNRSC  = 9999             THEN ASMNRSC  = . ;
IF ASSNRSC  = 9999             THEN ASSNRSC  = . ;
IF ASMMAT01 = 999              THEN ASMMAT01 = . ;
IF ASMMAT02 = 999              THEN ASMMAT02 = . ;
IF ASMMAT03 = 999              THEN ASMMAT03 = . ;
IF ASMMAT04 = 999              THEN ASMMAT04 = . ;
IF ASMMAT05 = 999              THEN ASMMAT05 = . ;
IF ASSSCI01 = 999              THEN ASSSCI01 = . ;
IF ASSSCI02 = 999              THEN ASSSCI02 = . ;
IF ASSSCI03 = 999              THEN ASSSCI03 = . ;
IF ASSSCI04 = 999              THEN ASSSCI04 = . ;
IF ASSSCI05 = 999              THEN ASSSCI05 = . ;

%PUT INFO: assign variable labels to recoded variables ;
LABEL
;

RUN ;

%let i = %eval(&i + 1);
%end;

%mend doit;

%doit;



