**********  SAS Input Program for SAS Version for the PC  **********;
*NOTE: This SAS program converts raw-data files into SAS version 6.12
       or 8.x system files

* I M P O R T A N T :
       PLEASE NOTE THAT YOU MUST SPECIFY THE "<PATH>" FOR BOTH THE
       INPUT AND OUTPUT FILES

OPTION NOSOURCE REPLACE NOCENTER ;

%let cty = ARM AUS BFL BGR BHR BWA COT CQU CHL CYP
           EGY ENG BSQ EST GHA HKG HUN IDN IRN ISR
           ITA JOR JPN KOR LBN LTU LVA MAR MDA MKD
           MYS NLD NOR NZL PSE PHL ROM RUS SAU SCO
           SGP SVK SVN SWE SYR TUN TWN USA UIN SCG
           ZAF;

*NOTE: LIBNAME and FILENAME statements need to be modified ;

* library where the SAS data files are written to;
***YOU NEED TO ADAPT THE FOLLOWING LINE;
LIBNAME libdat "<PATH>" ;

%macro doit;
   %let i = 1;
      %do %while(%length(%scan(&cty,&i)));
	     %let ccode = %lowcase(%scan(&cty,&i));

%let fname= BST&ccode.m3;

*NOTE: file-specification of the input raw-data file ;
***YOU NEED TO ADAPT THE FOLLOWING LINE;
FILENAME rawinp "<PATH>\&fname..RAW" ;

%PUT INFO: data set definition ;
%PUT INFO: data set definitions have to be modified ;

*Output data set ;
data libdat.&fname;

*NOTE: LRECL may need to be modified ;
INFILE rawinp LRECL=1250 END=eof LENGTH=lnline LINE=ln MISSOVER ;

%PUT INFO: shorten variable definitions to save secondary storage area ;
%PUT INFO: refer to the SAS manual for an appropriate length for variables ;

*NOTE: The LENGTH statement in a data step specifies the number of bytes, the
       SAS - sytem uses for storing values of variables in each data set being created. ;
LENGTH DEFAULT=5 ;
LENGTH      IDCNTRY    5 ;
LENGTH      IDPOP      3 ;
LENGTH      IDSCHOOL   6 ;
LENGTH      IDCLASS    8 ;
LENGTH      IDSTUD     8 ;
LENGTH      IDTEACH    8 ;
LENGTH      IDLINK     3 ;
LENGTH      IDSTRATE   3 ;
LENGTH      IDSTRATI   3 ;
LENGTH      IDGRADE    3 ;
LENGTH      IDGRADER   3 ;
LENGTH      ITCOURSE   3 ;
LENGTH      MATSUBJ    3 ;
LENGTH      SCISUBJ    3 ;
LENGTH      NMTEACH    3 ;
LENGTH      NSTEACH    3 ;
LENGTH      NTEACH     3 ;
LENGTH      INTMS03    3 ;
LENGTH      INTMS99    3 ;
LENGTH      INTMS95    3 ;
LENGTH      MATWGT     8 ;
LENGTH      SCIWGT     8 ;
LENGTH      TCHWGT     8 ;
LENGTH      TOTWGT     8 ;
LENGTH      JKREP      3 ;
LENGTH      JKZONE     3 ;
LENGTH      BSSNRSC    8 ;
LENGTH      BSMNRSC    8 ;
LENGTH      BSMMAT01   8 ;
LENGTH      BSMMAT02   8 ;
LENGTH      BSMMAT03   8 ;
LENGTH      BSMMAT04   8 ;
LENGTH      BSMMAT05   8 ;
LENGTH      BSSSCI01   8 ;
LENGTH      BSSSCI02   8 ;
LENGTH      BSSSCI03   8 ;
LENGTH      BSSSCI04   8 ;
LENGTH      BSSSCI05   8 ;
LENGTH      DPCDATE    6 ;


%PUT INFO: Missing category A refers to "not administered" values ;
%PUT INFO: Missing category I refers to "invalid" values ;
%PUT INFO: Missing category B refers to "logically not applicable" values ;
%PUT INFO: Missing category R refers to "not reached" items ;
%PUT INFO: Missing category . refers to missing values ;
MISSING A I B R ;

*NOTE: The INPUT statement reads raw data and assigns variable names.
       The positions specified below correspond to the positions of the variables
       in the raw data file. ;

INPUT
  IDCNTRY       1-   5     
  IDPOP         6-   6     
  IDSCHOOL      7-  12     
  IDCLASS      13-  20     
  IDSTUD       21-  30     
  IDTEACH      31-  38     
  IDLINK       39-  40     
  IDSTRATE     41-  43     
  IDSTRATI     44-  46     
  IDGRADE      47-  48     
  IDGRADER     49-  49     
  ITCOURSE     50-  51     
  MATSUBJ      52-  52     
  SCISUBJ      53-  53     
  NMTEACH      54-  54     
  NSTEACH      55-  55     
  NTEACH       56-  56     
  INTMS03      57-  57     
  INTMS99      58-  58     
  INTMS95      59-  59     
  MATWGT       60-  72 .6
  SCIWGT       73-  85 .6
  TCHWGT       86-  98 .6
  TOTWGT       99- 111 .6
  JKREP       112- 112     
  JKZONE      113- 114     
  BSSNRSC     115- 122 .3
  BSMNRSC     123- 130 .3
  BSMMAT01    131- 136 .2
  BSMMAT02    137- 142 .2
  BSMMAT03    143- 148 .2
  BSMMAT04    149- 154 .2
  BSMMAT05    155- 160 .2
  BSSSCI01    161- 166 .2
  BSSSCI02    167- 172 .2
  BSSSCI03    173- 178 .2
  BSSSCI04    179- 184 .2
  BSSSCI05    185- 190 .2
  DPCDATE     191- 196     
;


%PUT INFO: assign labels to variables ;
LABEL
  IDCNTRY  = "*COUNTRY ID*"
  IDPOP    = "*POPULATION ID*"
  IDSCHOOL = "*SCHOOL ID*"
  IDCLASS  = "*CLASS ID*"
  IDSTUD   = "*STUDENT ID*"
  IDTEACH  = "*TEACHER ID*"
  IDLINK   = "*TEACHER LINK NUMBER*"
  IDSTRATE = "EXPLICIT STRATUM CODE"
  IDSTRATI = "IMPLICIT STRATUM CODE"
  IDGRADE  = "*GRADE*"
  IDGRADER = "*STANDARDIZED GRADE ID*"
  ITCOURSE = "*NAME OF COURSE*"
  MATSUBJ  = "*MATHEMATICS TEACHER LINK*"
  SCISUBJ  = "*SCIENCE TEACHER LINK*"
  NMTEACH  = "*NUMBER OF MATH TEACHERS FOR STUDENT*"
  NSTEACH  = "*NUMBER OF SCIENCE TEACHERS FOR STUDENT*"
  NTEACH   = "*NO. OF MATH+SCIENCE TEACHERS COMBINED*"
  INTMS03  = "*IN 2003 ASSESSMENT POPULATION*"
  INTMS99  = "*IN 2003 AND 1999 ASSESSMENT POPULATION*"
  INTMS95  = "*IN 2003 AND 1999 AND 1995 ASS. POP*"
  MATWGT   = "*WEIGHT FOR MATHEMATICS TEACHER DATA*"
  SCIWGT   = "*WEIGHT FOR SCIENCE TEACHER DATA*"
  TCHWGT   = "*WEIGHT FOR MAT+SCI TEACHER DATA COMBIN*"
  TOTWGT   = "*STUDENT WEIGHT*"
  JKREP    = "*REPLICATE CODE*"
  JKZONE   = "*ZONE CODE*"
  BSSNRSC  = "*NATIONAL SCIENCE RASCH SCORE (150,10)*"
  BSMNRSC  = "*NATIONAL MATH RASCH SCORE (150,10)*"
  BSMMAT01 = "*1ST PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT02 = "*2ND PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT03 = "*3RD PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT04 = "*4TH PLAUSIBLE VALUE MATHEMATICS*"
  BSMMAT05 = "*5TH PLAUSIBLE VALUE MATHEMATICS*"
  BSSSCI01 = "*1ST PLAUSIBLE VALUE SCIENCE*"
  BSSSCI02 = "*2ND PLAUSIBLE VALUE SCIENCE*"
  BSSSCI03 = "*3RD PLAUSIBLE VALUE SCIENCE*"
  BSSSCI04 = "*4TH PLAUSIBLE VALUE SCIENCE*"
  BSSSCI05 = "*5TH PLAUSIBLE VALUE SCIENCE*"
  DPCDATE  = "*FILE CREATION DATE AT DPC HAMBURG*"
;

%PUT INFO: recode "not administered" data to missing code ".A" ;
*NOTE: If a CHECK variable is coded correctly with a BLANK, it will be recoded
       to "1", otherwise it will be recoded to "0" ;

IF IDCNTRY  = 99998            THEN IDCNTRY  = .A ;
IF IDPOP    = 8                THEN IDPOP    = .A ;
IF IDSCHOOL = 999998           THEN IDSCHOOL = .A ;
IF IDCLASS  = 99999998         THEN IDCLASS  = .A ;
IF IDSTUD   = 9999999998       THEN IDSTUD   = .A ;
IF IDTEACH  = 99999998         THEN IDTEACH  = .A ;
IF IDLINK   = 98               THEN IDLINK   = .A ;
IF IDSTRATE = 998              THEN IDSTRATE = .A ;
IF IDSTRATI = 998              THEN IDSTRATI = .A ;
IF IDGRADER = 8                THEN IDGRADER = .A ;
IF ITCOURSE = 98               THEN ITCOURSE = .A ;
IF INTMS03  = 8                THEN INTMS03  = .A ;
IF INTMS99  = 8                THEN INTMS99  = .A ;
IF INTMS95  = 8                THEN INTMS95  = .A ;
IF MATWGT   = 999998           THEN MATWGT   = .A ;
IF SCIWGT   = 999998           THEN SCIWGT   = .A ;
IF TCHWGT   = 999998           THEN TCHWGT   = .A ;
IF TOTWGT   = 999998           THEN TOTWGT   = .A ;
IF JKREP    = 8                THEN JKREP    = .A ;
IF JKZONE   = 98               THEN JKZONE   = .A ;
IF BSSNRSC  = 9998             THEN BSSNRSC  = .A ;
IF BSMNRSC  = 9998             THEN BSMNRSC  = .A ;
IF BSMMAT01 = 998              THEN BSMMAT01 = .A ;
IF BSMMAT02 = 998              THEN BSMMAT02 = .A ;
IF BSMMAT03 = 998              THEN BSMMAT03 = .A ;
IF BSMMAT04 = 998              THEN BSMMAT04 = .A ;
IF BSMMAT05 = 998              THEN BSMMAT05 = .A ;
IF BSSSCI01 = 998              THEN BSSSCI01 = .A ;
IF BSSSCI02 = 998              THEN BSSSCI02 = .A ;
IF BSSSCI03 = 998              THEN BSSSCI03 = .A ;
IF BSSSCI04 = 998              THEN BSSSCI04 = .A ;
IF BSSSCI05 = 998              THEN BSSSCI05 = .A ;

%PUT INFO: recode special missing codes to SAS missing codes ;
*NOTE: Special missing codes are recoded to SAS
       character - missing codes;


%PUT INFO: recode omitted data to missing code "." ;
*NOTE: BLANKS are recoded to SYSMIS by SAS ;

IF IDCNTRY  = 99999            THEN IDCNTRY  = . ;
IF IDPOP    = 9                THEN IDPOP    = . ;
IF IDSCHOOL = 999999           THEN IDSCHOOL = . ;
IF IDCLASS  = 99999999         THEN IDCLASS  = . ;
IF IDSTUD   = 9999999999       THEN IDSTUD   = . ;
IF IDTEACH  = 99999999         THEN IDTEACH  = . ;
IF IDLINK   = 99               THEN IDLINK   = . ;
IF IDSTRATE = 999              THEN IDSTRATE = . ;
IF IDSTRATI = 999              THEN IDSTRATI = . ;
IF IDGRADER = 9                THEN IDGRADER = . ;
IF ITCOURSE = 99               THEN ITCOURSE = . ;
IF INTMS03  = 9                THEN INTMS03  = . ;
IF INTMS99  = 9                THEN INTMS99  = . ;
IF INTMS95  = 9                THEN INTMS95  = . ;
IF MATWGT   = 999999           THEN MATWGT   = . ;
IF SCIWGT   = 999999           THEN SCIWGT   = . ;
IF TCHWGT   = 999999           THEN TCHWGT   = . ;
IF TOTWGT   = 999999           THEN TOTWGT   = . ;
IF JKREP    = 9                THEN JKREP    = . ;
IF JKZONE   = 99               THEN JKZONE   = . ;
IF BSSNRSC  = 9999             THEN BSSNRSC  = . ;
IF BSMNRSC  = 9999             THEN BSMNRSC  = . ;
IF BSMMAT01 = 999              THEN BSMMAT01 = . ;
IF BSMMAT02 = 999              THEN BSMMAT02 = . ;
IF BSMMAT03 = 999              THEN BSMMAT03 = . ;
IF BSMMAT04 = 999              THEN BSMMAT04 = . ;
IF BSMMAT05 = 999              THEN BSMMAT05 = . ;
IF BSSSCI01 = 999              THEN BSSSCI01 = . ;
IF BSSSCI02 = 999              THEN BSSSCI02 = . ;
IF BSSSCI03 = 999              THEN BSSSCI03 = . ;
IF BSSSCI04 = 999              THEN BSSSCI04 = . ;
IF BSSSCI05 = 999              THEN BSSSCI05 = . ;

%PUT INFO: assign variable labels to recoded variables ;
LABEL
;

RUN ;

%let i = %eval(&i + 1);
%end;

%mend doit;

%doit;



