* Define the data library ;
libname bm3 "D:\TIMSS2003\Data\SAS_Data" ;

* Include the macro to get the results ;
%include "D:\TIMSS2003\Programs\SAS_Programs\JackPV.sas" ;

* Get the math teacher data and select cases of interest ;
data teacher ;
     set bm3.BTSALLM3;
    select (BTBGAGE) ;
     when(1,2)   TCHAGE = 1 ;    /*Less than 30 years old*/
     when(3)     TCHAGE = 2 ;    /*30-39 years old       */
     when(4)     TCHAGE = 3 ;    /*40-49 years old       */
     when(5,6)   TCHAGE = 4 ;    /*50 years or older     */
     otherwise ;
    end ;

* Sort the science teacher data ;
proc sort data=teacher ;
     by IDCNTRY idteach idlink ;

* Get the student-teacher linkage data ;
data stdtch ;
     set bm3.BSTALLM3;

* Sort the student-teacher linkage data ;
proc sort data=stdtch ;
     by IDCNTRY idteach idlink ;

* Now merge the two files and select cases of interest ;
data merged3 ;
     merge stdtch teacher ;
     by IDCNTRY idteach idlink ;
     if SCIWGT > 0 and nmiss(TCHAGE) = 0 ;

* Define the format for the variables used ;
proc format library=work ;
 value country
    51 = 'ARMENIA'	 	36 = 'AUSTRALIA'				956 = 'BELGIUM FLEMISH'	  	100 = 'BULGARIA'
    48 = 'BAHRAIN'	 	72 = 'BOTSWANA'	       			9132 = 'CANADA ONTARIO'   	9133 = 'CANADA QUEBEC'
   152 = 'CHILE'		196 = 'CYPRUS'					818 = 'EGYPT'		  		926 = 'ENGLAND'        
  3724 = 'BASQUE'		233 = 'ESTONIA'					288 = 'GHANA'		  		344 = 'HONG KONG'       
   348 = 'HUNGARY'		360 = 'INDONESIA'				364 = 'IRAN'		  		376 = 'ISRAEL'   
   380 = 'ITALY'		400 = 'JORDAN'					392 = 'JAPAN'		  		410 = 'KOREA'   
   422 = 'LEBANON'		440 = 'LITHUANIA'				428 = 'LATVIA'		  		504 = 'MOROCCO'
   498 = 'MOLDOVA'		807 = 'MACEDONIA'				458 = 'MALAYSIA'	  		528 = 'NETHERLANDS'
   578 = 'NORWAY'		554 = 'NEW ZEALAND'				275 = 'PALESTINE'	  		608 = 'PHILIPPINES'
   642 = 'ROMANIA'		643 = 'RUSSIAN FEDERATION'		682 = 'SAUDI ARABIA'	  	927 = 'SCOTLAND'
   702 = 'SINGAPORE'	703 = 'SLOVAK REPUBLIC'			705 = 'SLOVENIA'	  		752 = 'SWEDEN'
   760 = 'SYRIA'    	788 = 'TUNISIA'					158 = 'CHINESE TAIPEI'	  	840 = 'UNITED STATES'            
   887 = 'YEMEN'      	891 = 'SERBIA'	710 = 'SOUTH AFRICA'		11800 = 'INDIANA US' ;     
  value age  1 = 'Less than 30 years old'
             2 = '30-39 years old'
             3 = '40-49 years old'
             4 = '50 years or older' ;

* Run the macro to get the results ; 
%JackPV (SCIWGT, jkzone, jkrep, 75, IDCNTRY TCHAGE, BSSSCI0, 5, merged3) ;

* Print the results ; 
proc print data=final noobs ;
     var IDCNTRY TCHAGE n SCIWGT mnpv mnpv_se pct pct_se ;
     format IDCNTRY country. TCHAGE age. n 6.0 SCIWGT 10.0 mnpv mnpv_se pct pct_se 6.2 ;

run ;

