* Computes Jackknife Standard Errors for DV;

%macro jackgen (wgt,jkz,jkr,njkz,cvar,dvar,fname);

* First count the number of classification variables and create vector with
  all but last classification variable;

%let i = 1;                            * Start counter at 1;
%do %while(%length(%scan(&cvar,&i)));
  %let i = %eval(&i + 1);
%end;
%let ncvar = %eval(&i-1);                * Diminish by 1 at the end;

%if &ncvar > 1 %then
%let cvar2 = %substr(&cvar,1,%eval(%length(&cvar) - %length(%scan(&cvar,&ncvar))));
%else %let cvar2 = ;

* Now create the data file with the replicate weights;
data a;
  set &fname (keep= &wgt &jkz &jkr &cvar &dvar);
  array rwt  rwt1 - rwt&njkz ;         * Replicate Weight     ;
  array wtx  wtx1 - wtx&njkz ;         * Weighted X           ;
  k = 1 ;                              * computes a constant  ;
   if &dvar >.Z;                       * Keep only non-missing;
  do i=1 to &njkz;
     if  &jkz ~= i             then rwt(i) = &wgt * 1;
     if (&jkz  = i & &jkr = 1) then rwt(i) = &wgt * 2;
     if (&jkz  = i & &jkr = 0) then rwt(i) = &wgt * 0;
   wtx(i)  = rwt(i) * &dvar  ;
  end;
    
    N = 1;
    wtx0  = &wgt * &dvar;                 * for the sum of X;

proc summary data=a missing;
     class k &cvar;
     var N &wgt rwt1-rwt&njkz wtx0 wtx1-wtx&njkz ;
     output out=b sum=;
     run;

proc summary data=b;
     var k _type_;
     output out=maxtype max=k maxtype;

data b;
     merge b maxtype (drop= _type_ _freq_);
     by k;
     maxtype2 = maxtype-1;

* Select the record which has the totals for &cvar2 and save to a file;
data type2  (keep = k &cvar2 t2wt t2rwt1-t2rwt&njkz);
     set b                        ;
     where _type_ = maxtype2      ;
     array rwt  rwt1-rwt&njkz     ;
     array t2rwt t2rwt1 - t2rwt&njkz;
     t2wt  = &wgt;
     do i = 1 to &njkz;
       t2rwt(i) = rwt(i);
     end;

proc sort data = b out=d;
     by k &cvar2;
     where _type_ = maxtype;

data e (drop = t2wt t2rwt1-t2rwt&njkz);
     merge d type2 ;
     by k &cvar2 ;
       twt = t2wt;
     array rwt   trwt1-trwt&njkz   ;
     array t2rwt t2rwt1-t2rwt&njkz ;
      do i= 1 to &njkz;
         rwt(i) = t2rwt(i);
      end;

data final (keep = N &cvar mnx mnx_se pct pct_se &wgt);
   set e;
   mnx  =  wtx0  / &wgt ;
   pct   = 100* (&wgt/ twt);
  array wtx  wtx1  - wtx&njkz  ;
  array rwt  rwt1  - rwt&njkz  ;
  array rmn  rmn1  - rmn&njkz  ;
  array trwt trwt1 - trwt&njkz ;
  array rp   rp1   - rp&njkz   ;
    mn_jvar = 0;
    p_jvar  = 0;
    n_jvar  = 0;
  do i = 1 to &njkz ;
    if rwt(i) > 0 then
       rmn(i)  =  wtx(i)  / rwt(i)            ; * Compute the mean for each replicate;
       else rmn(i)  = .                       ;
       rp(i)   = 100 * (rwt(i)  / trwt(i))    ; * Compute the proportion of each replicate;
    if rmn(i) <> . then
       mn_jvar = mn_jvar + (rmn(i)  - mnx )**2; * Compute JK_VAR for Mean ;
       p_jvar  = p_jvar  + (rp(i)   - pct )**2; * Compute JK_VAR for P ;
    if rwt(i)  > 0 then
       n_jvar = n_jvar   + (&wgt - rwt(i))**2; * Compute JK_VAR for N ;
  end;
    mnx_se  = sqrt(mn_jvar ) ;
    pct_se  = sqrt(p_jvar  ) ;
    n_se    = sqrt(n_jvar  ) ;
  run;

%mend jackgen;

