
*/ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * /*;
*/  Performs simple jackknife variance estimation for frequency counts and other /*;
*/  mean values involving plausible values.                                      /*;
*/ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * /*;

%macro jackpv (wgt,jkz,jkr,njkz,cvar,rootpv,npv,infile);

%let i = 1;                            * Start counter at 1;
%do %while(%length(%scan(&cvar,&i)));
  %let i = %eval(&i + 1);
%end;
%let ncvar = %eval(&i-1);                * Diminish by 1 at the end;

%if &ncvar > 1 %then
%let cvar2 = %substr(&cvar,1,%eval(%length(&cvar) - %length(%scan(&cvar,&ncvar))));
%else %let cvar2 = ;

* And generate the string with the PVs;
%let allpv = ;

%do i = 1 %to &npv;
  %let pv&i  = &rootpv.&i;
  %let allpv = &allpv &&pv&i ;
%end;

data a;
  set &infile (keep= &wgt &jkz &jkr &cvar &allpv) ;
  array rwt  rwt1  - rwt&njkz  ;         * Replicate Weight       ;
  array wtx  wtx1  - wtx&njkz  ;         * Weighted x             ;
  k = 1;
   if &pv1 > .z;
 * Here the replicate weights are computed;
  do i=1 to &njkz;
     if  &jkz ~= i             then rwt(i) = &wgt * 1;
     if (&jkz  = i & &jkr = 1) then rwt(i) = &wgt * 2;
     if (&jkz  = i & &jkr = 0) then rwt(i) = &wgt * 0;
   wtx(i)  = rwt(i) * &pv1 ;
  end ;
     wtx0  = &wgt * &pv1;
        N  = 1;

 * Compute the terms for the sum of X;
   %do i = 1 %to &npv;
     wtpv&i  = &wgt * &&pv&i ;
   %end;

proc summary data=a missing;
   class k &cvar;
   var N &wgt rwt1-rwt&njkz wtx0 wtx1-wtx&njkz
     %do i = 1 %to &npv;
        wtpv&i
     %end;
     ;
   output out=b sum=;
   run;

proc summary data=b;
     var k _type_;
     output out=maxtype max=k maxtype;

data b;
     merge b maxtype (drop= _type_ _freq_);
     by k;
     if k = 1;
     maxtype2 = maxtype-1;

* Select the record which has the totals for &cvar2 and save to a TEMP file;
data type2  (keep = k &cvar2 t2wt t2rwt1-t2rwt&njkz);
   set b                           ;
   where _type_ = maxtype2         ;
   array rwt  rwt1    - rwt&njkz   ;
   array t2rwt t2rwt1 - t2rwt&njkz ;
   t2wt  = &wgt ;
   do i = 1 to &njkz;
     t2rwt(i) = rwt(i);
   end;

proc sort data = b out = d;
   by k &cvar2;
   where _type_=maxtype;

data e (drop = t2wt t2rwt1-t2rwt&njkz);
   merge d type2 ;
   by k &cvar2 ;
    twt = t2wt ;
   array rwt   trwt1-trwt&njkz ;
   array t2rwt t2rwt1-t2rwt&njkz ;
    do i= 1 to &njkz;
      rwt(i) = t2rwt(i);
    end;

data final (keep = N &cvar &wgt  mnx     mnx_se
                   mnpv  mnpv_se mnx_jvr mnpv_ivr
                   pct pct_se );
   set e ;
   mnx   =  wtx0 / &wgt       ;
   pct   = 100 * (&wgt / twt) ;
     array wtx  wtx1  - wtx&njkz  ;
     array rwt  rwt1  - rwt&njkz  ;
     array rmnx rmnx1 - rmnx&njkz ;
     array trwt trwt1 - trwt&njkz ;
     array rp   rp1   - rp&njkz   ;
    mnx_jvr = 0 ;
    pct_var = 0 ;
  do i = 1 to &njkz ;
    if rwt(i) > 0 then
           rmnx(i) =wtx(i)/rwt(i)       ; * Compute the mean for each replicate;
           else rmnx(i) = .             ;
           rp(i) =100*(rwt(i)/trwt(i))  ; * Compute the proportion of each replicate;
    if rmnx(i) ~= . then
       mnx_jvr  = mnx_jvr  + (rmnx(i) - mnx  )**2  ; * Compute JK_Var for Mean X;
       pct_var  = pct_var  + (rp(i)   - pct)**2  ; * Compute JK_VAR for P ;
  end;
      %do i = 1 %to &npv;
         mnpv&i  =  wtpv&i  / &wgt ;
      %end;
    mnpv_ivr = 0;
    %if %eval(&npv) > 1 %then %do;
    mnpv_ivr = (1+(1 / %eval(&npv))) * var(of %do i = 1 %to &npv; mnpv&i %end;);
    %end;
    mnx_se   = sqrt(mnx_jvr);
    mnpv_se  = sqrt(mnx_jvr + mnpv_ivr);
    mnpv     = mean(of %do i = 1 %to &npv; mnpv&i  %end;);
    pct_se   = sqrt(pct_var);
  run;

%mend jackpv;
