* Computes Jackknife Standard Errors for Regression Coefficients    ;

%macro jackregP(wgt, jkz, jkr, njkz, cvar, xvar, rootpv, npv, infile);

* options mprint;

* WGT   : The sampling weight;
* JKZ   : The zone assignment;
* JKR   : Whether to zero or double the weights (1 or 0 only);
* NJKZ  : The number of Jackknife zones;
* CVAR  : The classification variable(s);
* XVAR  : The Independent variable(s);
* ROOTPV: The Root of the Plausible Values;
* NPV   : The number of Plausible values;
* INFILE: The file with the data;

* Files created: WORK.A     : Sorted file,
                 WORK.SSCP  : Sum of Squares and cross products,
                 WORK.SWEEPF: The file that is swept,
                 WORK.DESC  : Descriptive statistics,
                 WORK.REG   : Regression coefficients;

* First count the number of independent variables and create vector with
  the constant, the independent variable, and dependent variable;

%let i = 1;                            * Counts the Xvars;
%do %while(%length(%scan(&xvar,&i)));
  %let i = %eval(&i + 1);
%end;
%let nxvar = &i;                       * Includes the constant;

%let regvar  = k &xvar &rootpv;
%let nregvar = %eval(&nxvar + 1);      * Variables including Y and K;

%put &nxvar &regvar &nregvar;          * Just to check;

%let i = 1;                            * Count the Cvars;
%do %while(%length(%scan(&cvar,&i)));
  %let i = %eval(&i + 1);
%end;
%let ncvar    = %eval(&i-1);           * Diminish by 1 at the end;
%let lastcvar = %scan(&cvar,&ncvar);   * Last CVAR;

* Read the data file and keep only what is necessary sorting by the
  classification variables (CVAR);

proc sort data = &infile (keep= &wgt &jkz &jkr &cvar &rootpv.1 - &rootpv.&npv &xvar)
     out = a;
     by &cvar;
     where

        %let i = 1;
        %do %while(%length(%scan(&xvar &rootpv,&i)));

          %scan(&xvar &rootpv.1, &i) is not missing and

        %let i = %eval(&i + 1);
        %end;

        &wgt > 0;

* Now create the data file with the replicate weights and summarize;

%do pv = 1 %to &npv;

%let regvar  = k &xvar &rootpv&pv;

data sscp&pv (drop = &wgt &jkz &jkr k &xvar &rootpv&pv rwgt1-rwgt&njkz i

%do i = 1 %to &nregvar;
%if %eval(&i) < 11 %then %let ai=0%eval(&i-1);
 %else %let ai = %eval(&i-1);

  %do j = &i %to &nregvar;
  %if %eval(&j) < 11 %then %let bj=0%eval(&j-1);
   %else %let bj = %eval(&j-1);

  W&ai&bj._0 - W&ai&bj._&njkz

  %end;
  %end;);
  set a;
   by &cvar;

  array rwgt  rwgt1  - rwgt&njkz  ;         * Replicate Weight;

  * Computes the replicate weights;
  do i=1 to &njkz;
     if       &jkz ~= i             then rwgt(i) = &wgt * 1 ;
     else if (&jkz  = i & &jkr = 1) then rwgt(i) = &wgt * 2 ;
     else if (&jkz  = i & &jkr = 0) then rwgt(i) = &wgt * 0 ;
  end;

    K = 1;  * Create constant (vector of 1);

 * Define the arrays for the SSCP matrix;
%do i = 1 %to &nregvar;
%let a = %scan(&regvar,&i);

%if %eval(&i) < 11 %then %let ai=0%eval(&i-1);
 %else %let ai = %eval(&i-1);

  %do j = &i %to &nregvar;
  %let b = %scan(&regvar,&j);

  %if %eval(&j) < 11 %then %let bj=0%eval(&j-1);
   %else %let bj = %eval(&j-1);

%put;
%put Array for the product: &a * &b * Weight;
%put;

   array W&ai&bj  W&ai&bj._1 - W&ai&bj._&njkz;
   array S&ai&bj  S&ai&bj._1 - S&ai&bj._&njkz;

 %end;
 %end;

 * Accumulate here for the sums of square and cross products;

 * Initialize if the first case in last grouping variable;
    if first.&lastcvar then do;
       N = k;

%do i = 1 %to &nregvar;
%let a = %scan(&regvar,&i);

%if %eval(&i) < 11 %then %let ai=0%eval(&i-1);
 %else %let ai = %eval(&i-1);

  %do j = &i %to &nregvar;
  %let b = %scan(&regvar,&j);

  %if %eval(&j) < 11 %then %let bj=0%eval(&j-1);
   %else %let bj = %eval(&j-1);

   W&ai&bj._0 = &a * &b * &wgt; 
   S&ai&bj._0 = W&ai&bj._0 ;
    do z = 1 to &njkz;
     W&ai&bj(z) = &a * &b * rwgt(z);
     S&ai&bj(z) = W&ai&bj(z);
    end;

  %end;
  %end;

       end;

* Accumulate if not the last case for the group;

    if ~first.&lastcvar then do;
       N + k;

%do i = 1 %to &nregvar;
%let a = %scan(&regvar,&i);

%if %eval(&i) < 11 %then %let ai=0%eval(&i-1);
 %else %let ai = %eval(&i-1);

  %do j = &i %to &nregvar;
  %let b = %scan(&regvar,&j);

  %if %eval(&j) < 11 %then %let bj=0%eval(&j-1);
   %else %let bj = %eval(&j-1);

   W&ai&bj._0 = &a * &b * &wgt;
   S&ai&bj._0 +  W&ai&bj._0 ;
    do z = 1 to &njkz;
     W&ai&bj(z) = &a * &b * rwgt(z);
     S&ai&bj(z) +  W&ai&bj(z);
    end;

  %end;
  %end;

       end;

    if last.&lastcvar then output sscp&pv;

* Create the first matrix to be swept and add the elements below the diagonal;
data sweepf&pv;
 set sscp&pv;

%do i = 0 %to %eval(&nxvar-1);
%if %eval(&i) < 10 %then %let ai=0&i;
 %else %let ai = %eval(&i);

%do j = &i %to &nxvar;

%if %eval(&j) < 10 %then %let bj=0&j;
 %else %let bj = %eval(&j);

  array up&ai&bj s&ai&bj._0 - s&ai&bj._&njkz;
  array dn&bj&ai s&bj&ai._0 - s&bj&ai._&njkz;

  do z = 1 to %eval(&njkz+1);
    dn&bj&ai(z) = up&ai&bj(z);
  end;

%end;
%end;

%do sweep = 0 %to %eval(&nxvar-1);
%if %eval(&sweep) < 10 %then %let sweep=0&sweep;
 %else %let sweep = %eval(&sweep);

data sweepf&pv (drop = pivot0 - pivot&njkz);
  set sweepf&pv;

* Create the arrays for the sweeping;

array pivot pivot0 - pivot&njkz;

%do i = 0 %to &nxvar;
%if %eval(&i) < 10 %then %let ai=0&i;
 %else %let ai = %eval(&i);

%do j = 0 %to &nxvar;
%if %eval(&j) < 10 %then %let bj=0&j;
 %else %let bj = %eval(&j);

array s&ai&bj s&ai&bj._0 - s&ai&bj._&njkz;

%end;
%end;

 * Loop over all zones;
do z = 1 to %eval(&njkz+1);
 det      = 1.0 * s&sweep&sweep(z);
 pivot(z) = 1.0 / s&sweep&sweep(z);
 s&sweep&sweep(z) = pivot(z);

%do i = 0 %to &nxvar;
%if %eval(&i) < 10 %then %let ai=0&i;
 %else %let ai = %eval(&i);

%if %eval(&i) ~= %eval(&sweep) %then %do;

%do j = 0 %to &nxvar;
%if %eval(&j) < 10 %then %let bj=0&j;
 %else %let bj = %eval(&j);

 %if %eval(&j) ~= %eval(&sweep) %then %do;

 s&ai&bj(z) = s&ai&bj(z) - (pivot(z) * s&ai&sweep(z) * s&sweep&bj(z));

%end;
%end;
%end;
%end;

%do i = 0 %to &nxvar;
%if %eval(&i) < 10 %then %let ai=0&i;
 %else %let ai = %eval(&i);

%if %eval(&i) ~= %eval(&sweep) %then %do;
  s&sweep&ai(z) = 0.0 + s&sweep&ai(z) * pivot(z);
  s&ai&sweep(z) = 0.0 - s&ai&sweep(z) * pivot(z);
%end;
%end;

end;

* Compute the sums of squares;
%if &sweep = 0 %then %do;
  SS_Total = S&ai&bj._0;
  %end;

%if &sweep = %eval(&nxvar-1) %then %do;
  SS_Res   = S&ai&bj._0;
  SS_Reg   = SS_Total - SS_Res;
  Mult_R   = sqrt(1- SS_Res / SS_Total);
  Mult_RSQ = Mult_R**2;
  %end;

%end;

* Get the regression coefficients and their standard errors;
data reg&pv (keep = &cvar n SS_Res SS_Reg SS_Total Mult_RSQ rootpv pv
 %do i = 0 %to %eval(&nxvar-1);
 %if %eval(&i) < 10 %then %let ai=0&i;
  %else %let ai = %eval(&i);
   b&ai b&ai._jse
  %end;
   );
  set sweepf&pv;
   rootpv = "&rootpv";
   pv     = &pv;

 * Define the arrays here;

 %do i = 0 %to %eval(&nxvar-1);
 %if %eval(&i) < 10 %then %let ai=0&i;
  %else %let ai = %eval(&i);

 %if %eval(&nxvar) < 10 %then %let lastcol=0&nxvar;
  %else %let lastcol = %eval(&nxvar);
     array b_&ai s&ai&lastcol._1 - s&ai&lastcol._&njkz;

  b&ai = s&ai&lastcol._0;
  b&ai._jvar = 0;
   do z = 1 to &njkz;
    b&ai._jvar = b&ai._jvar + (b&ai - b_&ai(z))**2;
   end;
  b&ai._jse = sqrt(b&ai._jvar);

  %end;

%end;

data reg;
 set 
   %do pv = 1 %to &npv;
     reg&pv
   %end;
;
/*
proc print data= reg noobs;
     title "Regresion coefficients";
     format _numeric_ 10.4;
*/
data reg;
 set reg;
   %do i = 0 %to %eval(&nxvar-1);
   %if %eval(&i) < 10 %then %let ai=0&i;
   %else %let ai = %eval(&i);

 %if %eval(&nxvar) < 10 %then %let lastcol=0&nxvar;
  %else %let lastcol = %eval(&nxvar);

  if pv > 1 then b&ai._jse = .;

  %end;
/*
proc print data= reg noobs;
     title "Regresion coefficients";
     format _numeric_ 10.4;
*/
proc summary data=reg nway;
     class &cvar;
     var  %do i = 0 %to %eval(&nxvar-1);
          %if %eval(&i) < 10 %then %let ai=0&i;
          %else %let ai = %eval(&i);
            b&ai b&ai._jse
          %end;;
     output out = final 
         mean( n SS_Res SS_Reg SS_Total Mult_RSQ
          %do i = 0 %to %eval(&nxvar-1);
          %if %eval(&i) < 10 %then %let ai=0&i;
          %else %let ai = %eval(&i);
            b&ai b&ai._jse
          %end;) = n SS_Res SS_Reg SS_Total Mult_RSQ
              %do i = 0 %to %eval(&nxvar-1);
          %if %eval(&i) < 10 %then %let ai=0&i;
          %else %let ai = %eval(&i);
            b&ai b&ai._jse
          %end;

         var(%do i = 0 %to %eval(&nxvar-1);
          %if %eval(&i) < 10 %then %let ai=0&i;
          %else %let ai = %eval(&i);
            b&ai
          %end;) = 
              %do i = 0 %to %eval(&nxvar-1);
          %if %eval(&i) < 10 %then %let ai=0&i;
          %else %let ai = %eval(&i);
           vb&ai
          %end; ;

data reg;
     set final;
          %do i = 0 %to %eval(&nxvar-1);
          %if %eval(&i) < 10 %then %let ai=0&i;
          %else %let ai = %eval(&i);
            b&ai._ise = sqrt((1 + 1/&npv)*vb&ai);
            b&ai._se  = sqrt(b&ai._jse**2 + (1 + 1/&npv)*vb&ai);
          %end;;

run;
%mend jackregP;
