* set mprint=on.

* This macro computes regression coefficients and their statndard
*  errors jackknifed.
* The parameters needed to run this macro are:
*  INFILE :The datafile containing the information;
*  CVAR   :The list of classification variables;
*  XVAR   :The predictor variables;
*  NVAR   :The number of XVAR plus 1;
*  DVAR   :The "dependent variable" for which the regression
*          equation is computed;
*  NJKZ   :The Number of Jackknife replicates;
*  JKZ    :The variable with the assignment to JK zones;
*  JKR    :The variable with the JK Indicator (0 or 1).

* Here the macro begins.

set mprint=on.

define jackregP (infile  = !charend('/') /
                 cvar    = !charend('/') /
                 xvar    = !charend('/') /
                 rootpv  = !charend('/') /
                 npv     = !charend('/') /
                 njkz    = !charend('/') /
                 jkz     = !charend('/') /
                 jkr     = !charend('/') /
                 wgt     = !charend('/') ).

!onexpand

* Count the number of Variables.

!let !x = x
!do !i !in(!xvar)
!let !x = !concat(!x,"x")
!doend
!let !nvar = !length(!x)

!if (!length(!nvar)=1) !then !let !nvar2 = !concat("0",!nvar)
 !else !let !nvar2 = !nvar
  !ifend

* Obtain the number of replicates plus 1 (nr) for the sweep.

!let !nr = x
 !do !r = 1 !to !njkz
   !let !nr = !concat(!nr,x)
 !doend
!let !nr = !length(!nr)

* Loop over the 5 plausible values.
!do !pv = 1 !to !npv
!let !dvar = !concat(!rootpv,!pv).

get file = !infile.
weight off.

* A constant K is added to the file.
compute k = 1.

* The replicate weights are computed here.
compute rwgt0 = !wgt.
vector rwgt(!njkz).
loop #i = 1 to !njkz.
if (!jkz = #i and !jkr = 0)  rwgt(#i) = !wgt * 0.
if (!jkz = #i and !jkr = 1)  rwgt(#i) = !wgt * 2.
if (!jkz <>#i             )  rwgt(#i) = !wgt * 1.
end loop.

* Loop over the variables in the equation.
!let !xvar2 = !concat(k," ",!xvar," ",!dvar).
!do !i = 0 !to !nvar
  !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
  !else                     !let !ai = !i
  !ifend

!let !xa = !head(!xvar2)
!let !xvar2 = !tail(!xvar2)

*Select if not missing.
select if not(missing(!xa)).

!let !xvar3 = !concat(k," ",!xvar," ",!dvar)
!do !j = 0 !to !nvar
  !if (!length(!j)=1) !then !let !bj = !concat("0",!j)
  !else                     !let !bj = !j
  !ifend

!let !xb = !head(!xvar3)
!let !xvar3 = !tail(!xvar3).

* Multiply by the weight.
compute !concat(s,!ai,!bj,".0") = !xa * !xb * rwgt0.
vector !concat(s,!ai,!bj,".(",!njkz,")").

* Loop over the replicates.
loop #r = 1 to !njkz.
compute !concat(s,!ai,!bj,".(#r)") =  !xa * !xb * rwgt(#r).
end loop.

!doend
!doend

* execute.

* The main file is summarized here.

aggregate outfile = *
  / break = k !cvar
  / !do !i = 0 !to !nvar
     !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
     !else                     !let !ai = !i
     !ifend

    !do !j = 0 !to !nvar
     !if (!length(!j)=1) !then !let !bj = !concat("0",!j)
     !else                     !let !bj = !j
     !ifend

      !concat("  ",s,!ai,!bj,".0") to !concat("  ",s,!ai,!bj,".",!njkz)

    !doend
    !doend

    !concat(!blank(2),"  = sum(")

    !do !i = 0 !to !nvar
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend

    !do !j = 0 !to !nvar
      !if (!length(!j)=1) !then !let !bj = !concat("0",!j)
      !else                     !let !bj = !j
      !ifend
 
    !concat("  ",s,!ai,!bj,".0") to !concat("  ",s,!ai,!bj,".",!njkz)
    !doend
    !doend
    !concat(!blank(2)," )") / n = n(!wgt) .

execute.

    !do !i = 0 !to !nvar
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend

    !do !j = 0 !to !nvar
      !if (!length(!j)=1) !then !let !bj = !concat("0",!j)
      !else                     !let !bj = !j
      !ifend

vector !concat(s,!ai,!bj) = !concat(s,!ai,!bj,".0") to !concat(s,!ai,!bj,".",!njkz) .

    !doend
    !doend

* Implement the SWEEP operator here.
* Go over the number of replicates.
loop #r = 1 to !nr.

* SWEEP over the number of X variables.
!do !k = 0 !to !nvar
!if (!k<>!nvar) !then
   !if (!length(!k)=1) !then !let !ak = !concat("0",!k)
   !else                     !let !ak = !k
   !ifend

compute det = 1 * !concat(s,!ak,!ak,"(#r)").
compute pivot = 1.0 / !concat(s,!ak,!ak,"(#r)").
compute !concat(s,!ak,!ak,"(#r)") = pivot.

!do !j = 0 !to !nvar
!if (!j <> !k) !then
     !if (!length(!j)=1) !then !let !bj = !concat("0",!j)
     !else                     !let !bj = !j
     !ifend

!do !jp = 0 !to !nvar
!if (!jp <> !k) !then
     !if (!length(!jp)=1) !then !let !bjp = !concat("0",!jp)
     !else                      !let !bjp = !jp
     !ifend

compute #a=!concat(s,!bj,!bjp,"(#r)").
compute #b=(pivot*!concat(s,!bj,!ak,"(#r)")*!concat(s,!ak,!bjp,"(#r)")).
compute !concat(s,!bj,!bjp,"(#r)")=#a-#b.
!ifend
!doend
!ifend
!doend

!do !j = 0 !to !nvar
!if (!j <> !k) !then
     !if (!length(!j)=1) !then !let !bj = !concat("0",!j)
     !else                     !let !bj = !j
     !ifend

compute !concat(s,!ak,!bj,"(#r)") = 0 + !concat(s,!ak,!bj,"(#r)") * pivot.
compute !concat(s,!bj,!ak,"(#r)") = 0 - !concat(s,!bj,!ak,"(#r)") * pivot.
!ifend
!doend
!ifend

* Computes the sums of squares.

!if (!eval(!k) = 0) !then
do if #r = 1.
compute SS_Total = !concat(S,!bj,!bjp,".0") .
end if.
!ifend

!if (!k = !nvar) !then
do if #r = 1.
compute SS_Res   = !concat(S,!bj,!bjp,".0").
compute SS_Reg   = SS_Total - SS_Res.
compute Mult_R   = sqrt(1- SS_Res / SS_Total).
compute Mult_RSQ = Mult_R**2.
end if.
!ifend

!doend

end loop.

* Here the JK variance is computed.
!do !x = 0 !to !nvar
!if (!x <> !nvar) !then
     !if (!length(!x)=1) !then !let !ax = !concat("0",!x)
     !else                     !let !ax = !x
     !ifend

vector !concat(rb,!ax) = !concat(s,!ax,!nvar2,".1") to !concat(s,!ax,!nvar2,".",!njkz)
compute !concat(b,!ax) = !concat(s,!ax,!nvar2,".0").
compute !concat(b,!ax,".var") = 0.
loop #r = 1 to !njkz.
compute #b=((!concat(b,!ax)-!concat(rb,!ax,"(#r)"))**2).
compute !concat(b,!ax,".var")=!concat(b,!ax,".var")+ #b .
end loop.
compute !concat(b,!ax,".jse")  = sqrt(!concat(b,!ax,".var")).
!ifend
!doend

!let !cvar2 = !null
!do !cv !in(!cvar)
 !let !cvar2 = !concat(!cvar2," ",!cv," (label) ").
!doend

compute pv = !pv.

save outfile = !concat(reg,!pv).

!doend.

add files
  !do !pv = 1 !to !npv
    file = !concat(reg,!pv)
  !doend.
  .

    !do !i = 0 !to !nvar
    !if (!i <> !nvar) !then
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend

do if (pv > 1).
recode !concat("b",!ai,".jse")  (else= sysmis).
end if.
      !ifend
      !doend.

aggregate outfile = *
  / break = k !cvar
  / n SS_Reg SS_Total SS_Res Mult_RSQ 
      !do !i = 0 !to !nvar
      !if (!i <> !nvar) !then
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend
        !concat("b",!ai) !concat("b",!ai,".jse")
      !ifend
      !doend.
    = mean (
      n SS_Reg SS_Total SS_Res Mult_RSQ
      !do !i = 0 !to !nvar
      !if (!i <> !nvar) !then
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend
        !concat("b",!ai) !concat("b",!ai,".jse")
      !ifend
      !doend. 
      )  /
      !do !i = 0 !to !nvar
      !if (!i <> !nvar) !then
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend
        !concat("vb",!ai)
      !ifend
      !doend.
    = sd (
      !do !i = 0 !to !nvar
      !if (!i <> !nvar) !then
      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
      !else                     !let !ai = !i
      !ifend
        !concat("b",!ai) 
      !ifend
      !doend. 
  ).
   !do !i = 0 !to !nvar
   !if (!i <> !nvar) !then
   !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
   !else                     !let !ai = !i
   !ifend.

compute !concat("b",!ai,".ise")=  sqrt((1+1/!npv)*!concat("vb",!ai)**2).
compute !concat("b",!ai,".se") =sqrt(!concat("b",!ai,".jse")**2 + (1+1/!npv)*!concat("vb",!ai)**2).

   !ifend
   !doend.
 
save outfile = reg.

*title Predictors: !xvar / Predicted: !dvar.
*report format = list
*  / var !cvar2
*  !do !i = 0 !to !nvar
*  !if (!i <> !nvar) !then
*      !if (!length(!i)=1) !then !let !ai = !concat("0",!i)
*      !else                     !let !ai = !i
*      !ifend
*    !concat(!blank(2),"b",!ai) !concat(b,!ai,".se") !concat("b",!ai,".jse") !concat("b",!ai,".ise")
*  !ifend
*  !doend .

execute.

!enddefine.
